#include <apd.h>
#include <apd_internal.h>

// Report a function call stack for functions for which a threshold is greather or equal to
// the function importance.
void
_apd_report_function_stack( struct apd_object_struct *apd_object_internal, int threshold )
{
int loop;
int use_me_uninitialized, write_to_me;
struct apd_function_description_struct *function_info;

APD_INTERNAL_REPORT(5, "starting");
APD_INTERNAL_ASSERT((threshold >= -1*APD_IMPORTANCE_ABSOLUTE_LIMIT) 
                    && (threshold <= APD_IMPORTANCE_ABSOLUTE_LIMIT),
      "threshold must be with the range %d to %d.  threshold = %d",
      -1*APD_IMPORTANCE_ABSOLUTE_LIMIT, APD_IMPORTANCE_ABSOLUTE_LIMIT, threshold);
_apd_check_if_function_tracking_initialized(apd_object_internal);

// Check to make sure main apd_object contents still match backup copy like they should
APD_CHECK_DEBUG_OBJECT(apd_object_internal);

if (apd_object_internal->description_stack_size == 0)
  {
    fprintf(apd_object_internal->error_stream, 
            "_apd_report_function_stack(): warning: Empty function description stack \n");
    goto _apd_report_function_stack_end;
  }

// ouput the function description stack
fprintf(apd_object_internal->error_stream, "|||||  Function call stack  (depth = %d) |||||\n",
      apd_object_internal->description_stack_size);
for(loop = apd_object_internal->description_stack_size-1; loop >= 0; loop--)
  {
    function_info = (struct apd_function_description_struct *)
        g_slist_nth_data(apd_object_internal->description_stack, (uint)loop);
    APD_INTERNAL_ASSERT(function_info != NULL, 
                        "Function description stack is empty, loop=%d (in _apd_report_function_stack)", loop);

    if (threshold >= function_info->importance)
      {
        fprintf(apd_object_internal->error_stream, "%s\n", function_info->description);
      } // end if threshold met
  } // end for loop

// Trigger a Valgrind stack trace if needed
if (apd_object_internal->valgrind_stack_trace)
  {
    fprintf(apd_object_internal->error_stream, "======== Valgrind trace ========\n");
    write_to_me = use_me_uninitialized;
  }

_apd_report_function_stack_end:
fflush(apd_object_internal->error_stream);
APD_INTERNAL_REPORT(5, "ending");
} // end _apd_report_function_stack()
